<?php

namespace Teknisa\CADFIS\Services;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Request\DataSet;
use Teknisa\Libs\Util\XLS\FileLoader;
use Teknisa\Libs\Util\XLS\FileReader;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\XLS\Table;
use Teknisa\Libs\Util\XLS\Exception;
use Geral\Entities\Filial;
use Geral\Entities\Centcust;
use Zeedhi\Framework\DataSource;
use PHPExcel_Cell;
use PHPExcel_Shared_Date;
use Teknisa\CADFIS\Factories\Ger00000CadastramentosAutomaticoFactory;

class Ger00000CadastramentosAutomaticoService{ 
    
    
    protected $entityManager;
    protected $environment;
    private $titleType;
    protected $columns;
    protected $columnNames; //nome das colunas da planilha
    protected $optionalColumnNames; //nome das colunas da planilha que aceitam null
    protected $nrorg;
    protected $Ger00000CadastramentosAutomaticoFactory;
    
    const CD1_LENGTH = 1;
    const CD2_LENGTH = 2;
    const CD3_LENGTH = 3;
    const CD4_LENGTH = 4;
    const CD5_LENGTH = 5;
    const CD6_LENGTH = 6;
    const CD10_LENGTH = 10;
    const CD12_LENGTH = 12;
    const CD14_LENGTH = 14;
    const CD20_LENGTH = 20;
    const CD21_LENGTH = 21;
    const CD30_LENGTH = 30;
    const CD50_LENGTH = 50;
    const CD100_LENGTH = 100;
    
    public function __construct(Environment $environment, \Doctrine\ORM\EntityManager $entityManager, Ger00000CadastramentosAutomaticoFactory $Ger00000CadastramentosAutomaticoFactory) {
        $this->environment      = $environment;
        $this->entityManager    = $entityManager;
        $this->factory = $Ger00000CadastramentosAutomaticoFactory;
    }
    
    
    public function readXls($row){
        $file = $row['FILE'];
        $columnNames = $row['CAMPOS'];
        $sheetsToLoad = array('Sheasdasdsadet1', 'Sheet1', 'Planilha1');
        $xlsFile = FileLoader::loadFromBase64($file['content'], $file['name']);
        if(!$xlsFile){
            throw new \Exception(self::IMPORT_LOAD_FILE_ERROR);
        }
        $worksheet = $xlsFile->getActiveSheet();
        $table = FileReader::readWorksheetTable($worksheet, $columnNames);
        
        return $table;
    }
    
    public function formatXls(Table $table){
        $columnsReference = $table->getColumnsReference();
        $tableRows = $table->getRows();
        $columnsNumber = $table->getNumberOfColumns();
        $i = 0;
        foreach ($tableRows as &$row) {
            $row = $row['content'];
            for($column = 0; $column < $columnsNumber; $column++){
                $columnLetterIndex = PHPExcel_Cell::stringFromColumnIndex($column);
                $columnName = $columnsReference[$columnLetterIndex];
                $row[$columnName] = self::formatValue($row[$columnName], $columnName);
            }
            $row["ROWNUM"] = $i;

            $i++;
        }
        return $tableRows;
    }
    
    private function formatValue($value, $columnName){
        switch($columnName){
            case $columnName == 'CDCSTDIG1ENT':
            case $columnName == 'CDDIGVERIFINTFOR':    
            case $columnName == 'CDEMPRESA':
            case $columnName == 'IDCTBLCTREC':
            case $columnName == 'IDCTBLCRBANC':
            case $columnName == 'IDCTBLDBBANC':
            case $columnName == 'IDCTBAJCTPAG':
            case $columnName == 'IDAJUCTPAG':
            case $columnName == 'IDTPAJPAGAR':
            case $columnName == 'IDCTBAJCTREC':
            case $columnName == 'IDAJUCTREC':
            case $columnName == 'IDTPAJRECEBER':
                $value = self::leftZeroFill($value, self::CD1_LENGTH);
                break;
            
            case $columnName == 'CDMOTIPROR':
            case $columnName == 'CDMOTIPARC': 
            case $columnName == 'CDQUALIF':
            case $columnName == 'CDMODDOCFISC':
            case $columnName == 'CDFORMPGTO':
            case $columnName == 'CDTIPCOB':
            case $columnName == 'CDTPBAIXAPAG':
            case $columnName == 'IDIDENTCLIE':
            case $columnName == 'IDNATUCONT':
                $value = self::leftZeroFill($value, self::CD2_LENGTH);
                break;
            case $columnName == 'CDTIPOFORN':
            case $columnName == 'CDGRUPOFISC':
            case $columnName == 'CDTIPORECE':
            case $columnName == 'NRDIASADMI':
                $value = self::leftZeroFill($value, self::CD3_LENGTH);
                break;
            case $columnName == 'CDTPDEBIBANC':
            case $columnName == 'CDHISTPADR':
            case $columnName == 'CDTPCREDBANC':
            case $columnName == 'CDAJUCTPAG':
            case $columnName == 'CDAJUCTREC':
            case $columnName == 'CDCFOPFORN':
            case $columnName == 'CDPAIS':
            case $columnName == 'CDMUNICIPIO':
            case $columnName == 'CDLOGRADOURO':
            case $columnName == 'CDVIATURA':
            //case $columnName == 'CDFILIAL':
            case $columnName == 'CDFILCENTRALFATU':
            case $columnName == 'CDFILIOLAGETERC':
            case $columnName == 'CDMUNICIPIO':
                $value = self::leftZeroFill($value, self::CD4_LENGTH);
                break;
            case $columnName == 'CDSEGMER':
                $value = self::leftZeroFill($value, self::CD5_LENGTH);
                break;
            case $columnName == 'CDCLASSFINA':
            case $columnName == 'CDTPCTPAGAR':
            case $columnName == 'CDTPCTRECEB':
                $value = self::leftZeroFill($value, self::CD6_LENGTH);
                break;
            case $columnName == 'CDGRUPFORN':
            case $columnName == 'CDRECEFORN':
                $value = self::leftZeroFill($value, self::CD10_LENGTH);
                break;
            case $columnName == 'CDOPERINCLUSAO':
            case $columnName == 'CDOPERULTATU':
            case $columnName == 'CDOPERGESTOR':
                $value = self::leftZeroFill($value, self::CD12_LENGTH);
                break;
            case $columnName == 'NRINSJURTRAN':
                  $value = self::leftZeroFill($value, self::CD14_LENGTH);
                break;
            case $columnName == 'CDREFLOTECTBTDB':
            case $columnName == 'CDREFLOTECTBTCB':
            case $columnName == 'CDREFLOTECTBTAP':
            case $columnName == 'CDREFLOTECTBTAR':
            //case $columnName == 'IDTIPOCONT': 
            //case $columnName == 'IDCONTCCUS':
                $value = self::leftZeroFill($value, self::CD20_LENGTH);
                break;
            // case $columnName == 'CDFORNECED':
            case $columnName == 'CDFORNERELAC':
            case $columnName == 'CDFORNERELACAF':
                $value = self::leftZeroFill($value, self::CD21_LENGTH);
                break;
            case $columnName == 'CDEXFORNECED':
                $value = self::leftZeroFill($value, self::CD30_LENGTH);
                break;
            case $columnName == 'CDSENHAFOR':
            case $columnName == 'CDSENHAAFFOR':
            case $columnName == 'CDSENHACRFOR':
                $value = self::leftZeroFill($value, self::CD50_LENGTH);
                break;
            case $columnName == 'CDINTEGRAVAB':
                $value = self::leftZeroFill($value, self::CD100_LENGTH);
                break;
            case $columnName[0] == 'D' && $columnName[1] == 'S':  
            case $columnName[0] == 'N' && $columnName[1] == 'M':  
            case $columnName == 'NRCEPFORN':
            case $columnName == 'NRTELEFORN':
            case $columnName == 'NRINSJURFORN':
            case $columnName == 'IDTPIJURFORN':
            case $columnName == 'SGESTADO':
            case $columnName == 'CDCONTCTBL':
            case $columnName == 'CDCLIENTE':
                $value = strval($value);
                break;
            case $columnName[0] == 'D' && $columnName[1] == 'T': //se for data
            case $columnName == 'USR_DTEXPXML':
            case $columnName == 'USR_DTCADACLIE':
                $value = self::formatData($value);
                break;
            case $columnName == 'CDFORNECED':
                $value = self::formataCdFornecedor(strval($value));
                break;
            case $columnName == 'CDARVPROD':
            case $columnName == 'CDPRODESTO':
                $value = self::cleanSpecialCharacters(strval($value));
                break;
            case $columnName == 'QTCNDPG1CLIE':
            case $columnName == 'QTCNDPG2CLIE':
            case $columnName == 'QTCNDPG3CLIE':
            case $columnName == 'QTCNDPG4CLIE':
            case $columnName == 'QTCNDPG5CLIE':
            case $columnName == 'QTCNDPG6CLIE':
            case $columnName == 'QTFREEZERCLIE':
            case $columnName == 'NRSEQRETIRPJ':
            case $columnName == 'NRSEQRETCSLL':
            case $columnName == 'NRPARCNEGOCIO':
            case $columnName == 'QTPARCCUST':
            case $columnName == 'QTPREVRECCLI':
            case $columnName == 'VRCAPINTFUT':
            case $columnName == 'VRPEMAXDESCL':
            case $columnName == 'QTIMPEXTRMES':
            case $columnName == 'NRNOTACLIE':
            case $columnName == 'VRFATUCLIE':
            case $columnName == 'NRMESRESCLIE':
            case $columnName == 'NRANORESCLIE':
            case $columnName == 'VRSALCNJCLIE':
            case $columnName == 'VRSALARICLIE':
            case $columnName == 'NRDIAVALPERS':
            case $columnName == 'VRPESUBCLIEN':
            case $columnName == 'VRPEFORPRECO':
            case $columnName == 'VRTESTNIVCLI':
            case $columnName == 'VRLIMCREDCLI':
            case $columnName == 'VRCAPITALCLI':
            case $columnName == 'VRPATLIQCLI':
            case $columnName == 'VRPEMORBCLI':
            case $columnName == 'VRPEMULBCLI':
            case $columnName == 'NRPARCPGTOCL':
            case $columnName == 'NRDIAPGFIXCL':
            case $columnName == 'VRPERPG1CLIE':
            case $columnName == 'VRPERPG2CLIE':
            case $columnName == 'VRPERPG3CLIE':
            case $columnName == 'VRPERPG4CLIE':
            case $columnName == 'VRPERPG5CLIE':
            case $columnName == 'VRPERPG6CLIE':
            case $columnName == 'VRPEMARGLUCR':
            case $columnName == 'VRPECOVENFIL':
            case $columnName == 'QTFUNCFILI':
            case $columnName == 'VRPEORCBLOQFIL':
            case $columnName == 'VRPEORCALERTFIL':
            case $columnName == 'NRORGINCLUSAO':
            case $columnName == 'NRORGULTATU':
            case $columnName == 'VRPERCORCOMFJAN':
            case $columnName == 'VRPERCORCOMFFEV':
            case $columnName == 'VRPERCORCOMFMAR':
            case $columnName == 'VRPERCORCOMFABR':
            case $columnName == 'VRPERCORCOMFMAI':
            case $columnName == 'VRPERCORCOMFJUN':
            case $columnName == 'VRPERCORCOMFJUL':
            case $columnName == 'VRPERCORCOMFAGO':
            case $columnName == 'VRPERCORCOMFSET':
            case $columnName == 'VRPERCORCOMFOUT':
            case $columnName == 'VRPERCORCOMFNOV':
            case $columnName == 'VRPERCORCOMFDEZ':
            case $columnName == 'NRDIAINIORCAGOP':
            case $columnName == 'CDNVPRODUTO':
            case $columnName == 'NMPRODUTO':
            case $columnName == 'SGUNIDADE':
            case $columnName == 'CDBARPRODUTO': 
            case $columnName == 'CDCEST': 
            case $columnName == 'CDCLASFISC':
            case $columnName == 'NMRAZSOCTRAN':    
            case $columnName == 'IDTPIJURTRAN':
            case $columnName == 'NMFANTTRAN':
            case $columnName == 'NRINSESTTRAN':
            case $columnName == 'DSENDETRAN':
            case $columnName == 'NMBAIRTRAN':
            case $columnName == 'NRCEPTRAN':
            case $columnName == 'SGESTADO':
            case $value == null:
                break;
            case $columnName == 'VRPESOUNID':
            case $columnName == 'VRFATOCONV': 
                if($value == null){
                    $value = '1';
                }    
                break;
            case $columnName == 'VRARREDCMP': 
            case $columnName == 'VRARREDREQ': 
            case $columnName == 'VRARREDEMPPR': 
            case $columnName == 'VRPEFATOCORR':
                if($value == null){
                    $value = '0';
                }
                break;
            case $columnName == 'IDATIVO':
            case $columnName == 'IDCOMPARQXMLTRA':
                 if($value == null){
                    $value = 'S';
                }
                break;
            default: 
                $value = strval($value);
        }
        return $value;
    }
    
    private function formatData($data){
            $data =  date('d/m/Y', PHPExcel_Shared_Date::ExcelToPHP($data + 1.0));
        return $data;
    }
    
    private function leftZeroFill($string, $length){
        $string = str_pad(strval($string), $length, '0', STR_PAD_LEFT); 
        return $string;
    }
    
    private function formataCdFornecedor($string){
        /*if(strlen($string) == 14){
            if($string[0] != 'J')
                $string = 'J'.$string;
        }else if(strlen($string) == 11){
            if($string[0] != 'F')
                $string = 'F'.$string;
        }else{
            if($string[0] != 'I')
                $string = 'I'.$string;
        }*/
        return $string;
    }
    
    private function validaFatoConv($row, $mask){
        if(strlen($this->cleanSpecialCharacters($mask)) == strlen($row['CDARVPROD'])){
            if ($row['CDPRODESTO'] != $row['CDARVPROD'])
                return empty($row['VRFATOCONV']);
        }
        return false;
    }
    
    private function validaClassFisc($row){
        $result  = (!isset($row['CDCLASFISC']) || strlen($row['CDCLASFISC']) == 0)? false : $this->factory->verificaClassFis($row['CDCLASFISC']);
        
        
        if(!isset($row['CDCLASFISC']) || strlen($row['CDCLASFISC']) == 0)
            return false;
        else
            return $result? false: true;
    }
    
    public function verificaDado($dataSets){
        try{
            $data = $dataSets[0];
            $primaryKey = $dataSets[1][0]['primaryKey'];
            $table = $dataSets[1][0]['table_name'];
            $fields = $dataSets[2];
            $dataSetError = array();
            $i = 0;
            
            foreach($data as &$row){
                $aux = array();
                
                if($table=='CLIENTE_ENDECLIE'){
                    $existe = $this->factory->verificaDadoClie($row);
                }else if($table=='FORNECEDOR_ENDEFORN'){
                    $existe = $this->factory->verificaDadoForn($row);
                }else if($table=='CENTCUST_CENTCUSTFIL'){
                    $existe = $this->factory->verificaCentCust($row);
                }else 
                {
                    $existe = $this->factory->verificaDado($row, $primaryKey, $table);
                }
                
                if($table=='PRODUTO'){
                    $nrorg = $this->environment->getNrOrgTrab();
                    $mascaraProdutos = $this->factory->getMascaraProduto($nrorg);
                    $arrayMascara    = explode(".", $mascaraProdutos);
                    
                    
                    $erroMascaraProd = $this->validaMascaraProd($row, $arrayMascara);
                    if($erroMascaraProd){
                        $aux['IDREF'] = $i; 
                        $aux['DESCERROR'] = 'O nível do produto não corresponde ao Código de Árvore. de valor: '.$row[$primaryKey];
                        array_push($dataSetError, $aux);
                    }
                    
                    $erroProdEstoque = $this->validaProdEstoque($row, $data, $arrayMascara, $mascaraProdutos);
                    if($erroProdEstoque){
                        $aux['IDREF'] = $i; 
                        $aux['DESCERROR'] = 'Código do produto de estoque '.$row['CDPRODESTO'].' não existe ou preenchimento incorreto. de valor: '.$row[$primaryKey];
                        array_push($dataSetError, $aux);
                    }
                    
                    $errorFatoConv = $this->validaFatoConv($row, $mascaraProdutos);
                    if($errorFatoConv){
                        $aux['IDREF'] = $i; 
                        $aux['DESCERROR'] = 'Produto de compra sem Fator de Conversão. de valor: '.$row[$primaryKey];
                        array_push($dataSetError, $aux);
                    }
                    
                    $errorClassFisc = $this->validaClassFisc($row);
                    if($errorClassFisc){
                        $aux['IDREF'] = $i; 
                        $aux['DESCERROR'] = 'Classificação fiscal do produto (NCM) não cadastrada no sistema. de valor: '.$row[$primaryKey];
                        array_push($dataSetError, $aux);
                    }
                        
                }
                
                if($existe){
                    $aux['IDREF'] = $i; 
                    $aux['DESCERROR'] = 'Código já existente da coluna '.$primaryKey.' de valor: '.$row[$primaryKey];
                    array_push($dataSetError, $aux);
                }
                
                foreach($fields as $field){
                    if($field['COLUMN_NAME']=='IDCTBLCTREC' || $field['COLUMN_NAME']=='IDCTBLCRBANC' || $field['COLUMN_NAME']=='IDCTBLDBBANC' 
                    || $field['COLUMN_NAME']=='IDCTBAJCTPAG' || $field['COLUMN_NAME']=='IDCTBAJCTREC')
                        $field['DATA_LENGTH']=4;
                    if($field['COLUMN_NAME']=='IDCTBLCTPAG' || $field['COLUMN_NAME']=='IDCTBLCTPAG'||$field['COLUMN_NAME']=='IDVERSALDCLI'
                    ||$field['COLUMN_NAME']=='IDUTSALDCLI'||$field['COLUMN_NAME']=='IDUTSUBCLIEN')
                        $field['DATA_LENGTH']=5;
                    if($field['COLUMN_NAME']=='IDAJUCTPAG' || $field['COLUMN_NAME']=='IDTPAJPAGAR' || $field['COLUMN_NAME']=='IDAJUCTREC' 
                    || $field['COLUMN_NAME']=='IDTPAJRECEBER')
                        $field['DATA_LENGTH']=12;
                    if($field['COLUMN_NAME']=='IDTPCTPAGAR'|| $field['COLUMN_NAME']=='IDTPCTRECEBER'||$field['COLUMN_NAME']=='IDPRIPUBCLIE'
                    || $field['COLUMN_NAME']=='IDENDEMISCLI'|| $field['COLUMN_NAME']=='IDENDCFOPCLI'||$field['COLUMN_NAME']=='IDCLIEATIV'
                    || $field['COLUMN_NAME']=='IDTPIJURCLIE'|| $field['COLUMN_NAME']=='IDTPENDECLIE'|| $field['COLUMN_NAME']=='IDTPENDECLIECOBR'
                    || $field['COLUMN_NAME']=='IDTPENDECLIEENTR'|| $field['COLUMN_NAME']=='IDTIPOCONT'|| $field['COLUMN_NAME']=='IDCONTCCUS'|| $field['COLUMN_NAME']=='CDCONTREDU'
                    || $field['COLUMN_NAME']=='IDTPIJURTRAN' || $field['COLUMN_NAME']=='IDCOMPARQXMLTRA' || $field['COLUMN_NAME']=='IDATIVO'
                    || $field['COLUMN_NAME']=='IDNATUCONT'|| $field['COLUMN_NAME']=='IDPADRREL'|| $field['COLUMN_NAME']=='NRINSESTCLIE')
                        $field['DATA_LENGTH']=20;
                    if($field['COLUMN_NAME']=='IDENDEMISFOR'|| $field['COLUMN_NAME']=='IDENDCFOPFOR'
                    || $field['COLUMN_NAME']=='IDTPIJURFORN'|| $field['COLUMN_NAME']=='IDPRIPUBFORN'
                    || $field['COLUMN_NAME']=='IDTPENDEFORN'|| $field['COLUMN_NAME']=='IDTPENDEFORNCOBR'|| $field['COLUMN_NAME']=='IDTPENDEFORNENTR')
                        $field['DATA_LENGTH']=24;
                    if($field['COLUMN_NAME']=='NMTIPORECE')
                        $field['DATA_LENGTH']=40;
                    if($field['COLUMN_NAME']=='CDFILIAL')
                        $field['DATA_LENGTH']=5000;
                    if($field['COLUMN_NAME']=='CDPRODESTO')
                        $field['DATA_LENGTH']=20;
                    if(strlen(strval($row[$field['COLUMN_NAME']])) > $field['DATA_LENGTH']){
                        $aux['IDREF'] = $i;
                        $aux['DESCERROR'] = 'Tamanho do campo '.$field['COLUMN_NAME'].' incorreto.';
                        array_push($dataSetError, $aux);
                    }
                    if($field['DATA_TYPE'] == 'DATE'){
                        if(!checkdate( substr($row[$field['COLUMN_NAME']],3,2), substr($row[$field['COLUMN_NAME']],0,2) , substr($row[$field['COLUMN_NAME']],6,4) )){
                            $aux['IDREF'] = $i; 
                            $aux['DESCERROR'] = 'Data  '.$row[$field['COLUMN_NAME']].' inválida.';
                            array_push($dataSetError, $aux);
                        };
                    }
                    if($field['COLUMN_NAME'] == $primaryKey){
                        if(strlen($row[$field['COLUMN_NAME']]) == 0){    
                            $aux['IDREF'] = $i; 
                            $aux['DESCERROR'] = 'Código inválido da coluna '.$primaryKey.' de valor: VAZIO.';
                            array_push($dataSetError, $aux);
                        }
                    }
                    
                    switch($table){
                        case 'FORNECEDOR':
                            array_push($dataSetError, $this->verificaFornecedor($field['COLUMN_NAME'], $row[$field['COLUMN_NAME']], $i));
                        break;
                        case 'CLIENTE_ENDECLIE':
                            if($field['COLUMN_NAME'] == 'IDENDEMISCLI'|| $field['COLUMN_NAME'] =='IDENDCFOPCLI'){
                                if($row[$field['COLUMN_NAME']]=='0'||($row[$field['COLUMN_NAME']]!='Endereço do Cliente'&&$row[$field['COLUMN_NAME']]!='Endereço de Cobrança'&&$row[$field['COLUMN_NAME']]!='Endereço do Exterior'&&
                                $row[$field['COLUMN_NAME']]!='Endereço de Exportação'&&$row[$field['COLUMN_NAME']]!='Endereço de Entrega'&&$row[$field['COLUMN_NAME']]!='Endereço da Inscrição Estadual')){
                                    $aux['IDREF'] = $i;
                                    $aux['DESCERROR'] = 'A coluna '.$field['COLUMN_NAME'].' só aceita os valores de P, C, E, X, T, I.';
                                    array_push($dataSetError, $aux);
                                }
                                
                            }else
                            if($field['COLUMN_NAME'] == 'NRINSESTCLIE'){
                               $existe = $this->Ger00000CadastramentosAutomaticoFactory->verificaDadoNRINSESTCLIE($row[$field['COLUMN_NAME']]);
                                if(!$existe){
                                    $aux['IDREF'] = $i;
                                    $aux['DESCERROR'] = 'A coluna '.$field['COLUMN_NAME'].' aceita apenas números.';
                                    array_push($dataSetError, $aux);
                                }
                            }else
                            if($field['COLUMN_NAME'] == 'IDVERSALDCLI'||$field['COLUMN_NAME'] == 'IDUTSALDCLI'){
                                if($row[$field['COLUMN_NAME']]=='0'||($row[$field['COLUMN_NAME']]!='Sim'&&$row[$field['COLUMN_NAME']]!='Não')){
                                    $aux['IDREF'] = $i;
                                    $aux['DESCERROR'] = 'A coluna '.$field['COLUMN_NAME'].' só aceita os valores de S, N.';
                                    array_push($dataSetError, $aux);
                                }
                            }else
                            if($field['COLUMN_NAME'] == 'IDCLIEATIV'){
                                if($row[$field['COLUMN_NAME']]=='0'||($row[$field['COLUMN_NAME']]!='Ativo'&&$row[$field['COLUMN_NAME']]!='Inativo')){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'A coluna '.$field['COLUMN_NAME'].' só aceita os valores de S, N.';
                                        array_push($dataSetError, $aux);
                                }
                            } else
                            if($field['COLUMN_NAME'] == 'IDPRIPUBCLIE'){
                                if($row[$field['COLUMN_NAME']]=='0'||($row[$field['COLUMN_NAME']]!='Privado'&&$row[$field['COLUMN_NAME']]!='Público')){
                                    $aux['IDREF'] = $i;
                                    $aux['DESCERROR'] = 'A coluna '.$field['COLUMN_NAME'].' só aceita os valores de V, B.';
                                    array_push($dataSetError, $aux);
                                }
                            }else if($field['COLUMN_NAME'] == 'IDCALCDVCLIE'){
                                if($row[$field['COLUMN_NAME']]=='0'||($row[$field['COLUMN_NAME']]!='1'&&$row[$field['COLUMN_NAME']]!='2'&&$row[$field['COLUMN_NAME']]!='3')){
                                    $aux['IDREF'] = $i;
                                    $aux['DESCERROR'] = 'A coluna '.$field['COLUMN_NAME'].' só aceita os valores de 1, 2, 3.';
                                    array_push($dataSetError, $aux);
                                }
                            } else if($field['COLUMN_NAME'] == 'IDTPENDECLIE' && $row[$field['COLUMN_NAME']]!='Principal'){
                                    $aux['IDREF'] = $i;
                                    $aux['DESCERROR'] = 'A coluna '.$field['COLUMN_NAME'].' só aceita os valores de P.';
                                    array_push($dataSetError, $aux);
                            } else if($field['COLUMN_NAME'] == 'IDTPENDECLIECOBR' && $row[$field['COLUMN_NAME']]!='Cobrança'){
                                    $aux['IDREF'] = $i;
                                    $aux['DESCERROR'] = 'A coluna '.$field['COLUMN_NAME'].' só aceita os valores de C.';
                                    array_push($dataSetError, $aux);
                            } else if($field['COLUMN_NAME'] == 'IDTPENDECLIEENTR' && $row[$field['COLUMN_NAME']]!='Endereço de Entrega'){
                                    $aux['IDREF'] = $i;
                                    $aux['DESCERROR'] = 'A coluna '.$field['COLUMN_NAME'].' só aceita os valores de T.';
                                    array_push($dataSetError, $aux);
                            } else if($field['COLUMN_NAME'] == 'IDTPIJURCLIE'){
                                    if($row[$field['COLUMN_NAME']]=='0'||($row[$field['COLUMN_NAME']] != 'Físico'&&$row[$field['COLUMN_NAME']] != 'Jurídico'
                                    &&$row[$field['COLUMN_NAME']] != 'Internacional'&&$row[$field['COLUMN_NAME']] != 'Outros')){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'A coluna '.$field['COLUMN_NAME'].' só aceita os valores de F, J, I, O.';
                                        array_push($dataSetError, $aux);
                                    }
                            }
                            
                        break;
                        
                        //fornecedor
                        case 'FORNECEDOR_ENDEFORN':
                            
                            if($field['COLUMN_NAME'] == 'NRINSJURFORN'){
                               $existe = $this->factory->verificaDadoNRINSJURFORN($row);
                                if($existe){
                                    $aux['IDREF'] = $i;
                                    $aux['DESCERROR'] = 'Código já existente na coluna '.$field['COLUMN_NAME'].' de valor: '.$row[$field['COLUMN_NAME']];
                                    array_push($dataSetError, $aux);
                                }
                            }else if($field['COLUMN_NAME'] == 'IDENDEMISFOR'|| $field['COLUMN_NAME'] =='IDENDCFOPFOR'){
                                if($row[$field['COLUMN_NAME']]=='0'||($row[$field['COLUMN_NAME']]!='Endereço do Fornecedor'&&$row[$field['COLUMN_NAME']]!='Endereço de Cobrança'&&$row[$field['COLUMN_NAME']]!='Endereço do Exterior'&&
                                $row[$field['COLUMN_NAME']]!='Endereço de Exportação'&&$row[$field['COLUMN_NAME']]!='Endereço de Entrega'&&$row[$field['COLUMN_NAME']]!='Endereço da Inscrição Estadual')){
                                    $aux['IDREF'] = $i;
                                    $aux['DESCERROR'] = 'A coluna '.$field['COLUMN_NAME'].' só aceita os valores de P, C, E, X, T, I.';
                                    array_push($dataSetError, $aux);
                                }
                            }else if($field['COLUMN_NAME'] == 'DSENDEFORN'){
                                if($row[$field['COLUMN_NAME']]==null || $row[$field['COLUMN_NAME']]==''){
                                    $aux['IDREF'] = $i;
                                    $aux['DESCERROR'] = 'É obrigatória a informação da descrição do endereço ('.$field['COLUMN_NAME'].')';
                                    array_push($dataSetError, $aux);
                                }
                            }else if($field['COLUMN_NAME'] == 'IDPRIPUBFORN'){
                                if($row[$field['COLUMN_NAME']]=='0'||($row[$field['COLUMN_NAME']]!='Privado'&&$row[$field['COLUMN_NAME']]!='Público')){
                                    $aux['IDREF'] = $i;
                                    $aux['DESCERROR'] = 'A coluna '.$field['COLUMN_NAME'].' só aceita os valores de V, B.';
                                    array_push($dataSetError, $aux);
                                }
                            }else if($field['COLUMN_NAME'] == 'IDTPENDEFORN' && $row[$field['COLUMN_NAME']]!='Principal'){
                                    $aux['IDREF'] = $i;
                                    $aux['DESCERROR'] = 'É obrigatória a informação de pelo menos um endereço ('.$field['COLUMN_NAME'].')';
                                    array_push($dataSetError, $aux);
                            } else if($field['COLUMN_NAME'] == 'IDTPENDEFORNCOBR' && $row[$field['COLUMN_NAME']]!='Cobrança'){
                                    $aux['IDREF'] = $i;
                                    $aux['DESCERROR'] = 'A coluna '.$field['COLUMN_NAME'].' só aceita os valores de C.';
                                    array_push($dataSetError, $aux);
                            } else if($field['COLUMN_NAME'] == 'IDTPENDEFORNENTR' && $row[$field['COLUMN_NAME']]!='Endereço de Entrega'){
                                    $aux['IDREF'] = $i;
                                    $aux['DESCERROR'] = 'A coluna '.$field['COLUMN_NAME'].' só aceita os valores de T.';
                                    array_push($dataSetError, $aux);
                            } else if($field['COLUMN_NAME'] == 'IDTPIJURFORN'){
                                    if($row[$field['COLUMN_NAME']]=='0'||($row[$field['COLUMN_NAME']] != 'Físico'&&$row[$field['COLUMN_NAME']] != 'Jurídico'
                                    &&$row[$field['COLUMN_NAME']] != 'Internacional'&&$row[$field['COLUMN_NAME']] != 'Outros')){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'A coluna '.$field['COLUMN_NAME'].' só aceita os valores de F, J, I, O.';
                                        array_push($dataSetError, $aux);
                                    }
                            }
                            
                        break;
                        
                        case 'FILIAL':
                            if($field['COLUMN_NAME'] != 'CDFILIAL'){
                                array_push($dataSetError, $this->verificaFilial($field['COLUMN_NAME'], $row[$field['COLUMN_NAME']], $i));
                                
                            }
                        break;
                        case 'TIPOCTPAGAR':
                            switch($field['COLUMN_NAME']){
                                case 'IDCTBLCTPAG':
                                    if($row[$field['COLUMN_NAME']]!='Sim' && $row[$field['COLUMN_NAME']]!='Não'){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Valor do campo IDCTBLCTPAG deveria ser S ou N';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'IDTPCTPAGAR':
                                    if($row[$field['COLUMN_NAME']]!='Normal'&&$row[$field['COLUMN_NAME']]!='Adiantamento'&&$row[$field['COLUMN_NAME']]!='Empréstimo'){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Valor esperado no campo IDTPCTPAGAR é 1, 2 ou 3';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDCONTCTBL':
                                    $existe = $this->factory->verificaPrimaryKey($field['COLUMN_NAME'], $row[$field['COLUMN_NAME']], 'CONTCTBL');
                                    if(!$existe){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Não existe em CONTCTBL o '.$field['COLUMN_NAME'].' de valor: '.$row[$field['COLUMN_NAME']];
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDHISTPADR':
                                    $existe = $this->factory->verificaPrimaryKey($field['COLUMN_NAME'], $row[$field['COLUMN_NAME']], 'HISTPADR');
                                    if(!$existe){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Não existe em HISTPADR o '.$field['COLUMN_NAME'].' de valor: '.$row[$field['COLUMN_NAME']];
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDCLASSFINA':
                                    $existe = $this->factory->verificaPrimaryKey($field['COLUMN_NAME'], $row[$field['COLUMN_NAME']], 'CLASSFINANC');
                                    if(!$existe){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Não existe em CLASSFINANC o '.$field['COLUMN_NAME'].' de valor: '.$row[$field['COLUMN_NAME']];
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                            }
                        break;
                        case 'TIPOCTRECEBER':
                            switch($field['COLUMN_NAME']){
                                case 'IDCTBLCTREC':
                                    if($row[$field['COLUMN_NAME']]!='Sim' && $row[$field['COLUMN_NAME']]!='Não'){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Valor do campo IDCTBLCTREC deveria ser S ou N';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'IDTPCTRECEBER':
                                    if($row[$field['COLUMN_NAME']]!='Normal'&&$row[$field['COLUMN_NAME']]!='Adiantamento'){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Valor esperado no campo IDTPCTRECEBER é 1 ou 2';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDCONTCTBL':
                                    $existe = $this->factory->verificaPrimaryKey($field['COLUMN_NAME'], $row[$field['COLUMN_NAME']], 'CONTCTBL');
                                    if(!$existe){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Não existe em CONTCTBL o '.$field['COLUMN_NAME'].' de valor: '.$row[$field['COLUMN_NAME']];
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDCONTCTBLCNC':
                                    $existe = $this->factory->verificaPrimaryKey('CDCONTCTBL', $row[$field['COLUMN_NAME']], 'CONTCTBL');
                                    if(!$existe){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Não existe em CONTCTBL o '.$field['COLUMN_NAME'].' de valor: '.$row[$field['COLUMN_NAME']];
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDHISTPADR':
                                    $existe = $this->factory->verificaPrimaryKey($field['COLUMN_NAME'], $row[$field['COLUMN_NAME']], 'HISTPADR');
                                    if(!$existe){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Não existe em HISTPADR o '.$field['COLUMN_NAME'].' de valor: '.$row[$field['COLUMN_NAME']];
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDCLASSFINA':
                                    $existe = $this->factory->verificaPrimaryKey($field['COLUMN_NAME'], $row[$field['COLUMN_NAME']], 'CLASSFINANC');
                                    if(!$existe){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Não existe em CLASSFINANC o '.$field['COLUMN_NAME'].' de valor: '.$row[$field['COLUMN_NAME']];
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                            }
                        break;
                        case 'HISTPADR':
                            if($row[$field['COLUMN_NAME']]==null || $row[$field['COLUMN_NAME']]=='0000'){
                                $aux['IDREF'] = $i;
                                $aux['DESCERROR'] = 'Campo '.$field['COLUMN_NAME'].' não pode ser nulo ou zerado';
                                array_push($dataSetError, $aux);
                            }
                        break;
                        case 'CENTCUST_CENTCUSTFIL':
                            switch($field['COLUMN_NAME']){
                                case 'CDCENTCUST':
                                    if(strlen($row[$field['COLUMN_NAME']]) > 4){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Campo '.$field['COLUMN_NAME'].' não pode ter mais de 4 digitos';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDFILIAL':
                                    
                                    $filiais = $row[$field['COLUMN_NAME']];
                                    $filiais = str_replace(' ', '', $filiais);
                                    // $countpont = 0;
                                    // $qtdcorreta = 0;
                                    
                                    // for($x = 0; $x < strlen($filiais); $x++){
                                    //     if($filiais[$x] == ';'){
                                    //         $countpont++;
                                    //     }
                                    // }
                                    // $qtdfiliais = 0;
                                    // $qtdfiliais = strlen($filiais) - $countpont;
                                    // $qtdcorreta = ($countpont + 1) * 4;//trocar por split talvez
                                    $filiais = explode(";", $filiais);
                                    $erromaior = false;
                                    $erromenor = false;
                                    for($x = 0; $x < count($filiais); $x++){
                                        $existe = $this->factory->verificaPrimaryKey($field['COLUMN_NAME'], $filiais[$x], 'CENTCUSTFIL');
                                        if(!$existe && $row[$field['COLUMN_NAME']]!='T' && $filiais[$x] != ""){
                                            $aux['IDREF'] = $i;
                                            $aux['DESCERROR'] = 'Não existe em '.$field['COLUMN_NAME'].' a Filial de valor: '.$filiais[$x];
                                            array_push($dataSetError, $aux);
                                        }
                                        $erromaior = (strlen($filiais[$x]) > 4 && $filiais[$x] != "");
                                        $erromenor = (strlen($filiais[$x]) < 4 && $filiais[$x] != "");
                                    }
                                    if($erromaior && $row[$field['COLUMN_NAME']]!='T'){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Campo '.$field['COLUMN_NAME'].' possui codigo com tamanho maior que 4 digitos';
                                        array_push($dataSetError, $aux);
                                    }else if($erromenor && $row[$field['COLUMN_NAME']]!='T'){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Campo '.$field['COLUMN_NAME'].' possui codigo com tamanho menor que 4 digitos';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                            }
                        break;
                        //conta contabil
                        case 'CONTCTBL':
                            switch($field['COLUMN_NAME']){
                                case 'CDCONTCTBL':
                                   $mask = $this->factory->verificaDadoCDPICTCCTB($row);
                                   $mask = $mask->getCdpictcctb();
                                   $tamanho = array();
                                   $tamanhopornivel = array();
                                   $countpont = 0;
                                   $countcaractere = 0;
                                   if($mask!=null && $mask!=""){
                                       for($x = 0; $x < strlen($mask); $x++){
                                            if($mask[$x] == '.'){
                                        		array_push($tamanho, $x-$countpont);
                                        		$countpont++;
                                        	}
                                        }
                                        array_push($tamanho, strlen($mask)-$countpont);
                                        $pornivel = explode(".", $mask);
                                        if(!in_array(strlen($row[$field['COLUMN_NAME']]), $tamanho)){
                                            $aux['IDREF'] = $i;
                                            $aux['DESCERROR'] = 'O valor '.$row[$field['COLUMN_NAME']].' se encontra com erro na coluna '.$field['COLUMN_NAME'];
                                            array_push($dataSetError, $aux);
                                        }
                                        for($x = 0; $x < count($pornivel); $x++){
                                            $countcaractere = $countcaractere + strlen($pornivel[$x]);
                                            if($x == $row['CDNVCONTCTBL']-1){
                                                if(($countcaractere) != strlen($row[$field['COLUMN_NAME']])){
                                            		$aux['IDREF'] = $i;
                                                    $aux['DESCERROR'] = 'Erro de nivel na coluna CDNVCONTCTBL, o nivel e o tamanho da CDCONTCTBL não coincidem';
                                                    array_push($dataSetError, $aux);
                                                }
                                        	}
                                        }
                                        //var_dump($countcaractere . ' / ' . strlen($row[$field['COLUMN_NAME']]));
                                    }
                                break;
                                case 'CDCONTANTE':
                                    if($row[$field['COLUMN_NAME']]==null || $row[$field['COLUMN_NAME']]==''){
                                        if($row['CDNVCONTCTBL'] != 1){
                                            $aux['IDREF'] = $i;
                                            $aux['DESCERROR'] = 'CDCONTANTE só pode ser nulo se CDNVCONTCTBL indicar nível 1';
                                            array_push($dataSetError, $aux);
                                        }
                                    }
                                break;
                                case 'IDTIPOCONT':
                                    if($row[$field['COLUMN_NAME']]!='Analitico' && $row[$field['COLUMN_NAME']]!='Sintetico'){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Valor do campo IDTIPOCONT deveria ser A ou S';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'IDCONTCCUS':
                                    if($row[$field['COLUMN_NAME']]!='Oficial'&&$row[$field['COLUMN_NAME']]!='Gerencial'&&$row[$field['COLUMN_NAME']]!='Oficial/Gerencial'){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Valor esperado no campo IDCONTCCUS é O, R ou G';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDCONTREDU':
                                    if($row[$field['COLUMN_NAME']]==''){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Campo '.$field['COLUMN_NAME'].' não pode ser nulo';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'NMCONTCTBL':
                                    if($row[$field['COLUMN_NAME']]=='' || $row[$field['COLUMN_NAME']]==null){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Campo '.$field['COLUMN_NAME'].' não pode ser vazio';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDNVCONTCTBL':
                                    if($row[$field['COLUMN_NAME']]==''){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Campo '.$field['COLUMN_NAME'].' não pode ser nulo';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'IDNATUCONT':
                                    if($row[$field['COLUMN_NAME']]!='Vazio' && $row[$field['COLUMN_NAME']]!='Ativo' && $row[$field['COLUMN_NAME']]!='Passivo' && $row[$field['COLUMN_NAME']]!='Patrimonio Liquido' && $row[$field['COLUMN_NAME']]!='Resultado'&& $row[$field['COLUMN_NAME']]!='Compensação'&& $row[$field['COLUMN_NAME']]!='Outras'){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Valor do campo '.$field['COLUMN_NAME'].' deveria ser 00, 01, 02, 03, 04, 05 ou 09';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'IDPADRREL':
                                    if($row[$field['COLUMN_NAME']]!='Ativo' && $row[$field['COLUMN_NAME']]!='Passivo' && $row[$field['COLUMN_NAME']]!='Despesa' && $row[$field['COLUMN_NAME']]!='Receita' && $row[$field['COLUMN_NAME']]!='Custo' && $row[$field['COLUMN_NAME']]!='Não tem'){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Valor do campo '.$field['COLUMN_NAME'].' deveria ser A, P, D, R, C ou N';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                            }
                            
                        break;
                        //tipo recebimento
                        case 'TIPORECE':
                            switch($field['COLUMN_NAME']){
                                case 'NMTIPORECE':
                                    if($row[$field['COLUMN_NAME']]=='' || $row[$field['COLUMN_NAME']]==null){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'O campo '.$field['COLUMN_NAME'].' não pode estar vazio';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'VRTAXAADMI':
                                    if($row[$field['COLUMN_NAME']]=='' || $row[$field['COLUMN_NAME']]==null){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'O campo '.$field['COLUMN_NAME'].' não pode estar vazio';
                                        array_push($dataSetError, $aux);
                                    }
                                    $valor = 0;
                                    $maximo = 100;
                                    $valor = str_replace(".", ",", $row[$field['COLUMN_NAME']]);
                                    $qtd = explode(",", $valor);
                                    
                                    if($valor > $maximo){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'O valor da taxa não pode ultrapassar 100% no campo '.$field['COLUMN_NAME'];
                                        array_push($dataSetError, $aux);
                                    }
                                    if(count($qtd)>1){
                                        $qtddecimais = strlen($qtd[1]);
                                        if($valor == $maximo && $qtd[1]>0){
                                            $aux['IDREF'] = $i;
                                            $aux['DESCERROR'] = 'O valor da taxa não pode ultrapassar 100% no campo '.$field['COLUMN_NAME'];
                                            array_push($dataSetError, $aux);
                                        }
                                        // if($qtddecimais>2){
                                        //     $aux['IDREF'] = $i;
                                        //     $aux['DESCERROR'] = 'Numero de casa decimais do campo '.$field['COLUMN_NAME'].' definido para apenas 2';
                                        //     array_push($dataSetError, $aux);
                                        // }
                                    }
                                    if(substr_count($valor, ',')>1){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Permitido o uso de apenas uma "," no campo '.$field['COLUMN_NAME'].' , confira a planilha de importação';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDTPCTRECEB':
                                    $existe = $this->factory->verificaDadoCDTPCTRECEB($row);
                                    if($row[$field['COLUMN_NAME']]==''){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Campo '.$field['COLUMN_NAME'].' não pode ser vazio';
                                        array_push($dataSetError, $aux);
                                    }else if(!$existe){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Código NÃO existente na coluna '.$field['COLUMN_NAME'].' de valor: '.$row[$field['COLUMN_NAME']];
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                
                                
                            }
                        break;
                        //tipo de credito bancario
                        case 'TPCREDBANC':
                            switch($field['COLUMN_NAME']){
                                case 'IDCTBLCRBANC':
                                    if($row[$field['COLUMN_NAME']]!='Sim' && $row[$field['COLUMN_NAME']]!='Não'){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Valor do campo IDCTBLCRBANC deveria ser S ou N';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDCONTCTBL':
                                    $existe = $this->factory->verificaPrimaryKey($field['COLUMN_NAME'], $row[$field['COLUMN_NAME']], 'CONTCTBL');
                                    if(!$existe){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Não existe em CONTCTBL o '.$field['COLUMN_NAME'].' de valor: '.$row[$field['COLUMN_NAME']];
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDHISTPADR':
                                    $existe = $this->factory->verificaPrimaryKey($field['COLUMN_NAME'], $row[$field['COLUMN_NAME']], 'HISTPADR');
                                    if(!$existe){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Não existe em HISTPADR o '.$field['COLUMN_NAME'].' de valor: '.$row[$field['COLUMN_NAME']];
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDCLASSFINA':
                                    $existe = $this->factory->verificaPrimaryKey($field['COLUMN_NAME'], $row[$field['COLUMN_NAME']], 'CLASSFINANC');
                                    if(!$existe){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Não existe em CLASSFINANC o '.$field['COLUMN_NAME'].' de valor: '.$row[$field['COLUMN_NAME']];
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                            
                            }
                        break;
                        //tipo de debito bancario
                        case 'TPDEBIBANC':
                            switch($field['COLUMN_NAME']){
                                case 'IDCTBLDBBANC':
                                    if($row[$field['COLUMN_NAME']]!='Sim' && $row[$field['COLUMN_NAME']]!='Não'){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Valor do campo IDCTBLDBBANC deveria ser S ou N';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDCONTCTBL':
                                    $existe = $this->factory->verificaPrimaryKey($field['COLUMN_NAME'], $row[$field['COLUMN_NAME']], 'CONTCTBL');
                                    if(!$existe){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Não existe em CONTCTBL o '.$field['COLUMN_NAME'].' de valor: '.$row[$field['COLUMN_NAME']];
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDHISTPADR':
                                    $existe = $this->factory->verificaPrimaryKey($field['COLUMN_NAME'], $row[$field['COLUMN_NAME']], 'HISTPADR');
                                    if(!$existe){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Não existe em HISTPADR o '.$field['COLUMN_NAME'].' de valor: '.$row[$field['COLUMN_NAME']];
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDCLASSFINA':
                                    $existe = $this->factory->verificaPrimaryKey($field['COLUMN_NAME'], $row[$field['COLUMN_NAME']], 'CLASSFINANC');
                                    if(!$existe){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Não existe em CLASSFINANC o '.$field['COLUMN_NAME'].' de valor: '.$row[$field['COLUMN_NAME']];
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                            
                            }
                        break;
                        //Tipo de Ajuste Títulos a Pagar
                        case 'TIPOAJPAGAR':
                            switch($field['COLUMN_NAME']){
                                case 'IDCTBAJCTPAG':
                                    if($row[$field['COLUMN_NAME']]!='Sim' && $row[$field['COLUMN_NAME']]!='Não'){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Valor do campo IDCTBAJCTPAG deveria ser S ou N';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'IDAJUCTPAG':
                                    if($row[$field['COLUMN_NAME']]!='Acréscimo' && $row[$field['COLUMN_NAME']]!='Desconto'){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Valor do campo IDAJUCTPAG deveria ser A ou D';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'IDTPAJPAGAR':
                                    if($row[$field['COLUMN_NAME']]!='Adiantamento' && $row[$field['COLUMN_NAME']]!='Normal'){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Valor do campo IDTPAJPAGAR deveria ser A ou N';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDCONTCTBL':
                                    $existe = $this->factory->verificaPrimaryKey($field['COLUMN_NAME'], $row[$field['COLUMN_NAME']], 'CONTCTBL');
                                    if(!$existe){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Não existe em CONTCTBL o '.$field['COLUMN_NAME'].' de valor: '.$row[$field['COLUMN_NAME']];
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDHISTPADR':
                                    $existe = $this->factory->verificaPrimaryKey($field['COLUMN_NAME'], $row[$field['COLUMN_NAME']], 'HISTPADR');
                                    if(!$existe){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Não existe em HISTPADR o '.$field['COLUMN_NAME'].' de valor: '.$row[$field['COLUMN_NAME']];
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                            
                            }
                        break;
                        //Tipo de Ajuste Títulos a Receber
                        case 'TIPOAJRECEBER':
                            switch($field['COLUMN_NAME']){
                                case 'IDCTBAJCTREC':
                                    if($row[$field['COLUMN_NAME']]!='Sim' && $row[$field['COLUMN_NAME']]!='Não'){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Valor do campo IDCTBAJCTREC deveria ser S ou N';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'IDAJUCTREC':
                                    if($row[$field['COLUMN_NAME']]!='Acréscimo' && $row[$field['COLUMN_NAME']]!='Desconto'){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Valor do campo IDAJUCTREC deveria ser A ou D';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'IDTPAJRECEBER':
                                    if($row[$field['COLUMN_NAME']]!='Adiantamento' && $row[$field['COLUMN_NAME']]!='Normal' && $row[$field['COLUMN_NAME']]!='Devolução'){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Valor do campo IDTPAJRECEBER deveria ser A, D ou N';
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDCONTCTBL':
                                    $existe = $this->factory->verificaPrimaryKey($field['COLUMN_NAME'], $row[$field['COLUMN_NAME']], 'CONTCTBL');
                                    if(!$existe){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Não existe em CONTCTBL o '.$field['COLUMN_NAME'].' de valor: '.$row[$field['COLUMN_NAME']];
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                                case 'CDHISTPADR':
                                    $existe = $this->factory->verificaPrimaryKey($field['COLUMN_NAME'], $row[$field['COLUMN_NAME']], 'HISTPADR');
                                    if(!$existe){
                                        $aux['IDREF'] = $i;
                                        $aux['DESCERROR'] = 'Não existe em HISTPADR o '.$field['COLUMN_NAME'].' de valor: '.$row[$field['COLUMN_NAME']];
                                        array_push($dataSetError, $aux);
                                    }
                                break;
                            
                            }
                        break;
                        
                    }
                }
                $i++;
            }
            return $dataSetError;
            
        }

        catch(\Exception $e){
            throw $e;
        }
        
    }
    
    /*public function verificaForeignKeyFornecedor(){
        
    }*/
    
    public function verificaFornecedor($campo, $value, $i){
        $aux = array();
        switch($campo) {
            case 'IDTPENDEFORN':
                if($value != 'P' && $value != 'C' && $value != 'E' && $value != 'X'){
                    $aux['IDREF'] = $i; 
                    $aux['DESCERROR'] = 'Código inválido da coluna '.$campo.'  de valor: '.$value.'.';
                    //array_push($dataSetError, $aux);
                }
                break;
            case 'IDTPIJURFORN':
                if($value != 'F' && $value != 'J' && $value != 'I' && $value != 'O'){
                    $aux['IDREF'] = $i; 
                    $aux['DESCERROR'] = 'Código inválido da coluna '.$campo.'  de valor: '.$value.'.';
                    //array_push($dataSetError, $aux);
                }
                break;
            case 'NRINSJURFORN':
                if($value == null){
                    $aux['IDREF'] = $i; 
                    $aux['DESCERROR'] = 'Código inválido da coluna '.$campo.'  de valor: '.$value.'.';
                    //array_push($dataSetError, $aux);
                }
                break;
        }
        
        return $aux;
        
    }
    
    public function verificaCliente($campo, $value, $i){
        switch($campo) {
            case 'IDTPENDECLIE':
                if($value != 'P' && $value != 'C' && $value != 'E' && $value != 'X' && $value != 'T' && $value != 'O' && $value != 'M' && $value != 'I'){
                    $aux['IDREF'] = $i; 
                    $aux['DESCERROR'] = 'Código inválido da coluna '.$campo.'  de valor: '.$value.'.';
                }
                break;
        }
        
        return $aux;
        
    }
    
    public function verificaFilial($campo, $value, $i){
        switch($campo) {
            case 'IDTPENDEFILI':
                if($value != 'P' && $value != 'C' && $value != 'E' && $value != 'X' && $value != 'T'){
                    $aux['IDREF'] = $i; 
                    $aux['DESCERROR'] = 'Código inválido da coluna '.$campo.'  de valor: '.$value.'.';
                }
                break;
        }
        
        return $aux;
        
    }
    
    public function cleanSpecialCharacters($string){
        $string = str_replace('.', '', $string); // Replaces all spaces with hyphens.
        return preg_replace('/[^A-Za-z0-9\-]/', '', $string); // Removes special chars.
    }
    
    public function validaProdEstoque($currentRow, $dataset, $arrMask, $mask){
        $mask = self::cleanSpecialCharacters($mask);
        //valida nivel produto com prodesto preenchido indevidamente
        if( ($currentRow['CDNVPRODUTO'] < count($arrMask) && !empty($currentRow['CDPRODESTO'])) )
            return true;
            
        //valida produto de nivel completo sem preenchimento de prodesto
        if( ($currentRow['CDNVPRODUTO'] == count($arrMask) && empty($currentRow['CDPRODESTO'])) )
            return true;
            
        //valida produto de estoque setado com tamanho incorreto
        $invalid = !empty($currentRow['CDARVPROD'])? (strlen($currentRow['CDARVPROD']) != strlen($mask)) : false;

        $boolProdEstoValid = $this->array_some($dataset->getArrayCopy(), function($row) use ($currentRow){
            //valida se o produto de estoque está sendo criado nesta mesma importação
            return !empty($currentRow['CDPRODESTO'])?  ($currentRow['CDPRODESTO'] == $row['CDARVPROD']) : true;
        });
        
        $nrorg = $this->environment->getNrOrgTrab();
        
        if(!$boolProdEstoValid){
            //Caso o produto de estoque nao esteja sendo criado neste lançamento confere se ele já existe no bd. caso nao exista retorna erro de validacao
            $bdProduct = $this->factory->getProdEstoBD($currentRow['CDPRODESTO'], $nrorg);
            if(empty($bdProduct))
                return true;
        }
        
        //torna falso para erros caso passe por todas validacoes
        return false;
    }
    
    public function validaMascaraProd($row, $mask){
        $codArvProd = $row['CDARVPROD'];
        $nivelProd = $row['CDNVPRODUTO'];
        $mascara = '';
        
        if(intval($nivelProd) > count($mask))
            return true;
            
        for($i = 0; $i < $nivelProd; $i++)
            $mascara = $mascara.$mask[$i];
       
        return !(strlen($codArvProd) == strlen($mascara));
    }
    
    public function atualizaCdproduto($dataSet){
        $nrorg = $this->environment->getNrOrgTrab();
        $mask = $this->factory->getMascaraProduto($nrorg);
        $arrMask = explode(".", $mask);


        //caso seja produto de estoque força fator de conversão = 1
        foreach($dataSet as &$row){
            if($row['CDPRODESTO'] == $row['CDARVPROD'])
                $row['VRFATOCONV'] = 1;
        }             
                    
        //atualiza cdproduto com novocodigo caso mascara de produto seja maior que 10
        if(strlen($this->cleanSpecialCharacters($mask)) > 10){
            foreach($dataSet as &$row){
                $newcode = $this->factory->newCodeProdEsto($nrorg);
                $row['CDPRODUTO'] = $newcode;
            }
            
            //atualiza cdprodesto com os códigos da novocodigo no lugar do arvprod (Somente se for produto de nivel completo)
            foreach($dataSet as &$row){
                //caso seja produto de compra
                if(!empty($row['CDPRODESTO'])){
                    //validar esta parte usando a base do teste q tem mascara maior de 10
                    $onDataset = array_filter($dataSet, function($validationRow) use(&$row){
                        return $row['CDPRODESTO'] == $validationRow['CDARVPROD'];
                    });
                    $filteredArr = array_values($onDataset);
                    if(!empty($onDataset)){
                        $row['CDPRODESTO'] = $filteredArr[0]['CDPRODUTO'];
                    }else{
                        $cdprodestoBd = $this->factory->getProdEstoBD($row['CDPRODESTO'], $nrorg)['CDPRODUTO'];
                        $row['CDPRODESTO'] = $cdprodestoBd;
                    }
                    
                }
            }
        }
        
        return $dataSet;    
    }
    
    public function insereAgrupamentos($dataset){
        foreach($dataset as $row){
            $this->factory->callInsereAgrupa($row['CDARVPROD']);
        }
    }
    
    public function array_some(array $array, callable $fn) {
        foreach ($array as $value) {
            if($fn($value)) {
                return true;
            }
        }
        return false;
    }
    
    public function saveContaContabil($data) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            
            $updates = $this->factory->getUpdatesContaContabil(null);
                        
            foreach($updates as $update){
                
                $this->factory->updateContaContabil($update['LINHA']);
            }
            
            $this->factory->deleteContaContabil();
            
             for($i = 0; $i < count($data); $i++){
                
                $contacontabil = $this->factory->insereContaContabil(
                    $data[$i]['CDCONTCTBL'], $data[$i]['CDCONTREDU'], $data[$i]['NMCONTCTBL'],
                    $data[$i]['CDNVCONTCTBL'], $data[$i]['CDCONTANTE'], $data[$i]['IDTIPOCONT'],
                    $data[$i]['IDNATUCONT'], $data[$i]['IDPADRREL'],
                    $data[$i]['IDCONTCCUS'], $data[$i]['CDCONTPLANREFE'], $data[$i]['CDCONTREFEPRE'],
                    $data[$i]['CDCONTREFEIMUISEN']
                );
                
                $this->entityManager->persist($contacontabil);
            }
            
            
            $this->entityManager->flush();
            
            $maior = $this->factory->getMaxContaContabil();
            
            $updates = $this->factory->getUpdatesContaContabil($maior['CDCONTCTBL']);
                        
            foreach($updates as $update){
                $this->factory->updateContaContabil($update['LINHA']);
            }
            
            $this->entityManager->flush();
            

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function validaEndereco($row){
        $erro = $this->factory->validaEndereco($row);
        return $erro;
    }   
    
    public function validaEnderecoCobr($row){
        $erro = $this->factory->validaEnderecoCobr($row);
        return $erro;
    }   
    
    public function validaEnderecoEntr($row){
        $erro = $this->factory->validaEnderecoEntr($row);
        return $erro;
    }  

    public function insertCliente($row) {
            
            try{
            $params = array(
            'P_CDCLIENTE' => "" . $row['IDTPIJURCLIE']. "" .$row['NRINSJURCLIE'],
            'P_NRINSJURCLIE'   =>  $row['NRINSJURCLIE'],
            'P_IDTPIJURCLIE'  =>  $row['IDTPIJURCLIE'],
            'P_NRINSESTCLIE'   =>  $row['NRINSESTCLIE'],
            'P_NMRAZSOCCLIE'  =>  $row['NMRAZSOCCLIE'],
            'P_NMFANTCLIE'    =>  $row['NMFANTCLIE'],
            'P_IDENDEMISCLI'  =>  $row['IDENDEMISCLI'],
            'P_IDENDCFOPCLI' =>  $row['IDENDCFOPCLI'], 
            'P_IDPRIPUBCLIE'  =>  $row['IDPRIPUBCLIE'],
            'P_IDVERSALDCLI'=>  $row['IDVERSALDCLI'],
            'P_IDUTSALDCLI'=>  $row['IDUTSALDCLI'],
            'P_IDCALCDVCLIE'  =>  $row['IDCALCDVCLIE'],
            'P_IDUTSUBCLIEN'   =>  $row['IDUTSUBCLIEN']
            );
            $this->factory->insertCliente($params);
            
            }   catch (\Exception $e) {
                
                throw $e;
    }
    }

}